"#import libs/utils.inc.src"

// Can create two limited users : ssh and ftp (when optional passwords are provided)
// Make sure to have created the custom remote and members accounts before that
// For the exact effects on permissions, see https://usine.solution-libre.fr/qtg/grey-hack/-/issues/9
// Will clear the log file and all /Config/ folders as an additional security in case the server got misused before
// !!! This program will self-delete after use !!!
// Note : chmodding takes a bit of time for unknown reasons (and GH doesn't show prints while running)
// Expect ONE MINUTE or so until the single print statement shows up
	
imports.temp = {}
imports.file = {}
	
imports.temp.AddCmd = function(arr, files)
	for file in files
		arr.push("/bin/"+file)
	end for
end function
imports.temp.AddSoft = function(arr, files)
	for file in files
		arr.push("/usr/bin/"+file+".exe")
	end for
end function

// #2.6: If "destructive security" is desired, delete /etc/passwd and the services in /server/ (to prevent a shutdown)
//imports.file.USELESS = ["/etc/passwd","/server"]
imports.file.USELESS = []
// #2.3: Nslookup, whois, Mail.exe aren't useful on a server, maaaybe on a dedicated proxy... delete it
// #2.4: At the time of writing, there's no obvious benefit with not deleting CodeEditor.exe and build
// #2.5: Are members allowed the use this server as a last-point proxy to reach a ftp? If not, delete the ftp command
imports.temp.AddCmd(imports.file.USELESS,["build","whois","nslookup","ftp"])
imports.temp.AddSoft(imports.file.USELESS,["CodeEditor", "Mail"])

// #1.3: Give execution rights to everyone for "/bin/clear"
imports.file.SAFE = []
imports.temp.AddCmd(imports.file.SAFE,["clear"])

// #3.3: Set group for "/bin/ssh" as "ssh" and give it group-level execution rights
imports.file.SSH = []
imports.temp.AddCmd(imports.file.SSH,["ssh"])
// Nothing.
imports.file.FTP = []

// #3.4: Set group "members" and give group-level execution rights for...
imports.file.ALLOWED = []
imports.temp.AddCmd(imports.file.ALLOWED,["cd","ls","pwd","cat","rm","mv","cp","mkdir","rmdir","touch"])
imports.temp.AddSoft(imports.file.ALLOWED,["Terminal","FileExplorer","Notepad"])

imports.temp = null

//command: setup
if not imports.utils.IsRoot() then imports.utils.Exit("Must be root.")

length = imports.utils.ParamLength(params)
if length > 2 or imports.utils.HasFlag(params, "h", "help") then exit("<b>Usage: "+imports.utils.GetProgName()+" [(opt) ssh password] [(opt) ftp password]</b>")
comp = get_shell.host_computer

sshPass = imports.utils.GetParam(params, 0)
ftpPass = imports.utils.GetParam(params, 1)

// Utility method : only delete the files, not the directories
// (Can be used on a file too)
WipeDir = function(path)
	dir = comp.File(path)
	if not dir then return
	if path[path.len-1] == "/" then
		for file in dir.get_files // NOT THE FOLDERS!!!!
			file.delete
		end for
	else
		dir.delete
	end if
end function
// Utility method : delete all elements in the dir, then delete the dir itself
FullWipe = function(file)
	if not file then return
	for child in file.get_files
		child.delete()
	end for
	for child in file.get_folders
		FullWipe(child)
	end for
	file.delete()
end function

// #0.1: users able to bounce through ssh
SSH_GROUP = "ssh"
// #0.2: users able to access the /Public/Downloads/ folder
FTP_GROUP = "ftp"
// #0.3: users connected to manipulate non-public files (subgroup)
MAIN_GROUP = "home"
// #0.4: users with a private space (inherits files group)
USER_GROUP = "members"

// #1.1 By default, removing all perms for everyone on everything
main = comp.File("/")
for mod in ["u-rwx", "g-rwx", "o-rwx"]
	main.chmod(mod,1)
end for
// #1.2 Give read access to everyone in /
main.chmod("o+r",0)

NewLimitedUser = function(user, pass, grp)
	if not comp.create_user(user, pass) then comp.change_password(user, pass)
	comp.create_group(user, grp)

	// #2.1: Delete the home folder of (...) + "ssh"/"ftp"-only users
	dir = comp.File("/home/"+user+"/")
	if not dir then return
	dir.delete()
	end if
end function
// #1.4 Create the groups and add them to users accordingly
if sshPass then NewLimitedUser("proxy", sshPass, SSH_GROUP)
if ftpPass then NewLimitedUser("files", ftpPass, FTP_GROUP)

SetRead = function(path, group)
	file = comp.File(path)
	if not file then return
	recur = 0
	file.set_group(group,recur)
	file.chmod("g+r",recur)
end function
// #3.1 Set group for /home/ as "files" and give it group-level read
SetRead("/home/", MAIN_GROUP)
// #1.5 Set group for /Public/ as "ftp" and give it group-level read
SetRead("/Public/", FTP_GROUP)

// #1.6 Set group for /Public/Downloads/ as "ftp" and give it (recursive) group-level read-write
ftpDir = comp.File("/Public/Downloads/")
if ftpDir then
	ftpDir.set_group(FTP_GROUP,1)
	ftpDir.chmod("g+rw", 1)
end if

users = []
for user in comp.File("/home/").get_folders
	if user.name == "guest" then
		// #2.1: Delete the home folder of guest + (...) users
		FullWipe(user)
	else
		// #2.2 Give (recursive) !user-level! read-write for child folders of /home/
		users = users.push(user.path)
		user.chmod("u+rw",1)
	end if
end for

for path in imports.file.USELESS
	WipeDir(path)
end for
	
// #2.2: It would be a good idea to delete/reset the /Config/ folder for all users, just in case. (...)
users = users.push("/root")
for user in users
	WipeDir(user+"/Config/")
end for

// The 15 following lines are applying the suggestions listed at the "imports.file." part
for path in imports.file.SAFE
	file = comp.File(path)
	file.chmod("o+x",1)
end for

Safe = function(group, paths)
	for path in paths
		file = comp.File(path)
		recur=1
		file.chmod("g+x",recur)
		file.set_group(group,recur)
	end for
end function
Safe(USER_GROUP, imports.file.ALLOWED)
Safe(SSH_GROUP, imports.file.SSH)
Safe(FTP_GROUP, imports.file.FTP) // Normally no-op

imports.utils.SelfDestruct()

// #2.2: It would be a good idea to delete/reset (...), just in case. and the log file too.
WipeDir("/var/system.log")
comp.touch("/var/", "system.log")

print("Setup is finished")